// Copyright (c) 2024 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include "qtwebengine/common/extensions/api/generated_schemas.h"

#include <algorithm>
#include <iterator>

#include "base/stl_util.h"

namespace {
constexpr char kResourcesPrivate[] = R"R({"functions":[{"parameters":[{"$ref":"resourcesPrivate.Component","name":"component"},{"type":"function","name":"callback","parameters":[{"additionalProperties":{"type":"any"},"type":"object","name":"result"}]}],"name":"getStrings","type":"function"}],"documentation_options":{},"deprecated":null,"namespace":"resourcesPrivate","events":[],"platforms":null,"internal":false,"properties":{},"types":[{"enum":[{"name":"identity"},{"name":"pdf"}],"type":"string","id":"resourcesPrivate.Component"}]})R";
constexpr char kWebrtcDesktopCapturePrivate[] = R"R({"functions":[{"parameters":[{"name":"request","$ref":"webrtcDesktopCapturePrivate.RequestInfo"},{"type":"function","name":"callback","parameters":[{"type":"string","name":"streamId"},{"name":"options","$ref":"webrtcDesktopCapturePrivate.Options"}]}],"returns":{"type":"integer","name":"chooseDesktopMedia"},"name":"chooseDesktopMedia","type":"function"}],"documentation_options":{},"deprecated":null,"namespace":"webrtcDesktopCapturePrivate","events":[],"platforms":null,"internal":false,"properties":{},"types":[{"type":"object","id":"webrtcDesktopCapturePrivate.RequestInfo","properties":{"guestProcessId":{"name":"guestProcessId","type":"integer"},"guestRenderFrameId":{"name":"guestRenderFrameId","type":"integer"}}},{"type":"object","id":"webrtcDesktopCapturePrivate.Options","properties":{"canRequestAudioTrack":{"name":"canRequestAudioTrack","type":"boolean"}}}]})R";
}  // namespace

namespace extensions {
namespace api {

// static
bool QtWebEngineGeneratedSchemas::IsGenerated(base::StringPiece name) {
  return !Get(name).empty();
}

// static
base::StringPiece QtWebEngineGeneratedSchemas::Get(base::StringPiece name) {
  static const struct kSchemaMapping {
    const base::StringPiece name;
    const base::StringPiece schema;
    constexpr bool operator<(const kSchemaMapping& that) const {
      return name < that.name;
    }
  } kSchemas[] = {
    {"resourcesPrivate", kResourcesPrivate},
    {"webrtcDesktopCapturePrivate", kWebrtcDesktopCapturePrivate},
  };
  auto it = std::lower_bound(std::begin(kSchemas), std::end(kSchemas),
    kSchemaMapping{name, base::StringPiece()});
  if (it != std::end(kSchemas) && it->name == name)
    return it->schema;
  return base::StringPiece();
}

}  // namespace api
}  // namespace extensions
